(function (factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['prefix-umd'], factory);
    } else if (typeof module === 'object' && module.exports) {
        // Node/CommonJS
        module.exports = factory(require('prefix-umd'));
    } else {
        // Browser globals
        window.SimpleSlider = factory(window.Prefix);
    }
}(function (Prefix) {

    //兼容性处理
    function whichTransitionEvent(){
        var t;
        var el = document.createElement('div');
        var transitions = {
            'transition':'transitionend',
            'OTransition':'oTransitionEnd',
            'MozTransition':'transitionend',
            'WebkitTransition':'webkitTransitionEnd',
            'MsTransition':'msTransitionEnd'
        };
        for(t in transitions){
            if( el.style[t] !== undefined ){
                return transitions[t];
            }
        }
    }
    var transitionend = whichTransitionEvent();
    var transformProperty = Prefix.prefix('transform');
    var transitionProperty = Prefix.prefix('transition');
    var transformDash = Prefix.dash('transform');

    /**
     * 自定义toumove事件时需要阻止设备默认的touchmove事件，比如页面滑动，否则会出现一些莫名其妙的bug；
     * 同时还避免slide和页面同时滑动的不太好的体验。
     * @param slideSelector 容器选择器
     * @param pageSelector 选项选择器
     * @param initIndex 初始轮播序列号
     * @param initPosition 初始位置
     * @param width 选项宽度
     * @param gap 选项间隔
     * @param currentClass 当前选项标识类名
     * @param duration 过渡动画时长/秒
     * @param isAutoScroll 自动滚动
     * @param autoTime 自动滚动间隔时间
     * @param autoDirect 自动滚动方向
     * @param autoHeightInit 高度自适应内容功能的初始化逻辑
     * @param autoHeightMove 高度自适应内容功能的其它逻辑
     * @param animateStartFunc 过渡开始（执行组件不相关逻辑）
     * @param atEndFunc 滚动到最后一个选项
     * @param atStartFunc 滚动到第一个选项
     * @param isPreventDefault 是否阻止滑动事件默认行为（在可滑动页面中不建议禁止，在不可滑动页面中建议禁止）
     */
    function SimpleSlider(params){
        this.touchstart = false;
        this.preventDefault = false;//是否阻止页面滑动
        this.isFixed = true;//自动滑动是否完成
        this.isPreventDefault = params.isPreventDefault!=null?params.isPreventDefault:false;

        this.duration = params.duration?params.duration:0.4;
        this.currentClass = params.currentClass?params.currentClass:'current-item';
        this.isAutoScroll = params.isAutoScroll?params.isAutoScroll:false;
        this.autoTime = params.autoTime?params.autoTime:5000;
        this.autoDirect = params.autoDirect?params.autoDirect:1;
        this.autoHeightMove = params.autoHeightMove?params.autoHeightMove:function(lt,lh,rt,rh,h,tr,mh){};
        this.autoHeightInit = params.autoHeightInit?params.autoHeightInit:function(h,mh,tr){}
        this._autoHeight = (params.autoHeightMove&&params.autoHeightInit)?true:false;
        this.animateStartFunc = params.animateStartFunc?params.animateStartFunc:function(){};
        this.atEndFunc = params.atEndFunc?params.atEndFunc:function(){
            return this.index==(this.slideLength-1)
        };
        this.atStartFunc = params.atStartFunc?params.atStartFunc:function(){
            return this.index==0;
        };

        this.pageWidth = params.width>0?params.width:document.documentElement.clientWidth;
        this.pageGap = params.gap>0?params.gap:0;
        this.pageWidth += 2*this.pageGap;

        this.initIndex = params.initIndex?params.initIndex:0;
        this.initPosition = params.initPosition?params.initPosition:-this.initIndex*this.pageWidth;
        this.index = this.initIndex;

        this.$slide = document.querySelector(params.slideSelector);
        this.$slidePage = document.querySelectorAll(params.pageSelector);
        this.slideLength = this.$slidePage.length;
        this.$slidePage[this.index].classList.add(this.currentClass);
    }

    //自动滚动
    SimpleSlider.prototype.autoScroll = function(){
        var that = this;
        if(that.autoInterval){
            clearInterval(that.autoInterval);
        }
        that.autoInterval = setInterval(function(){
            if(that.index>=(that.slideLength-1)){
                that.autoDirect = -1;
            }
            if(that.index<=0){
                that.autoDirect = 1;
            }
            if(that.autoDirect>0){
                that.nextItem();
            }else{
                that.prevItem();
            }
        },that.autoTime);
    }

    //流程结束
    SimpleSlider.prototype.removeListener = function(){
        /**
         * caution
         * 考虑监听然后解绑的流程应该是考虑如果初始化该组件过多会导致页面有太多监听的缘故吧......
         */
        this.$slide.removeEventListener('touchmove',this.moveEvent,false);
        this.$slide.removeEventListener('touchend',this.endEvent,false);
    }

    SimpleSlider.prototype.init = function(){
        var self = this;
        if(this.slideLength>1){
            //每屏宽度等于屏幕宽度
            this.$slide.style.width = this.slideLength*this.pageWidth+'px';
            var currentHeight = 0;
            for(var i=0;i<this.slideLength;i++){
                var item = this.$slidePage[i];
                item.style.width = (this.pageWidth-2*this.pageGap)+'px';
                item.style.marginLeft = this.pageGap+'px';
                item.style.marginRight = this.pageGap+'px';

                if(this._autoHeight){//记录初始高度
                    var style = window.getComputedStyle(item);
                    var height = parseFloat(style.getPropertyValue('height'));
                    if(!this.maxHeight){
                        this.maxHeight = height;
                    }else if(height>this.maxHeight){
                        this.maxHeight = height;
                    }
                    if(item.classList.contains(this.currentClass)){
                        currentHeight = height;
                    }
                    if(i==this.slideLength-1){
                        this.autoHeightInit(currentHeight,this.maxHeight,'none');
                    }
                    item.setAttribute('init-height',height);
                }
            }
            this.startX = 0;
            this.startY = 0;
            this.moveX = 0;
            this.moveY = 0;

            this.$slide.addEventListener(transitionend,function(){
                if(!self.isFixed){
                    self.isFixed = true;
                    self.startX = 0;
                    self.startY = 0;
                    self.moveX = 0;
                    self.moveY = 0;
                    self.$slidePage[self.index].classList.add(self.currentClass);
                    self.removeListener();
                }
            });

            this.$slide.addEventListener('touchstart',function(e){
                if(self.isFixed){
                    var touch = e.touches[0];
                    self.cssTransform(self.$slide);
                    self.touchstart = true;
                    self.startX = touch.clientX;
                    self.startY = touch.clientY;
                    /* caution */
                    self.moveEvent = self.move.bind(self);
                    self.endEvent = self.end.bind(self);
                    self.$slide.addEventListener('touchmove',self.moveEvent);
                    self.$slide.addEventListener('touchend',self.endEvent);
                }
                e.stopPropagation();
                if(this.isPreventDefault){
                    e.preventDefault();
                }
            });

            if(this.initIndex){
                this.cssTransform(this.$slide,-this.initIndex*this.pageWidth);
                this.animateStartFunc();
            }

            if(this.isAutoScroll){
                this.autoScroll();
            }
        }
    }

    SimpleSlider.prototype.moveItem = function(e){
        e.preventDefault();//阻止默认滑动
        var absX = Math.abs(this.moveX);
        if(this.atStartFunc()&&this.moveX>0){
            //第一幕不能再向左移动
        }else if(this.atEndFunc()&&this.moveX<0){
            //最后一幕不能再向右移动
        }else{
            if(absX>=parseInt(this.pageWidth/10)){//超过阀值，触发自动移动
                this.isFixed = false;
                this.$slidePage[this.index].classList.remove(this.currentClass);
                if(this.moveX<0){
                    this.index = this.index+1;//向右边
                    this.autoDirect = 1;
                }else{
                    this.index = this.index-1;//向左边
                    this.autoDirect = -1;
                }
                this.animateStartFunc();
                this.cssTransform(this.$slide,-this.index*this.pageWidth,this.duration);
            }else{
                this.cssTransform(this.$slide,-this.index*this.pageWidth+this.moveX);
            }
        }
    }

    //向右侧滑动-下一栏
    SimpleSlider.prototype.nextItem = function(){
        if(this.index<(this.slideLength-1)){
            this.autoDirect = 1;
            this.isFixed = false;//置为非，防止touchstart、touchmove、touchend事件逻辑执行
            this.$slidePage[this.index].classList.remove(this.currentClass);
            this.index = this.index+1;
            this.animateStartFunc();
            this.cssTransform(this.$slide,-this.index*this.pageWidth,this.duration);
        }
    }

    //向左侧滑动-上一栏
    SimpleSlider.prototype.prevItem = function(){
        if(this.index>0){
            this.autoDirect = -1;
            this.isFixed = false;//置为非，防止touchstart、touchmove、touchend事件逻辑执行
            this.$slidePage[this.index].classList.remove(this.currentClass);
            this.index = this.index-1;
            this.animateStartFunc();
            this.cssTransform(this.$slide,-this.index*this.pageWidth,this.duration);
        }
    }

    SimpleSlider.prototype.cssTransform = function(target,x,second){
        if(x!=null){
            if(this._autoHeight){

                var len = Math.abs(x);
                var leftNo = parseInt(len/this.pageWidth);
                var rightNo = leftNo+1;

                var rightX = (len - this.pageWidth*leftNo)/this.pageWidth;
                var leftX = 1-rightX;
                var transition = 'none';

                if(second!=null){
                    if(this.autoDirect == 1){//右移进位
                        leftNo = leftNo-1;
                        rightNo = leftNo+1;

                        var tempX = leftX;
                        leftX = rightX;
                        rightX = tempX;
                    }
                    transition = transformDash+' '+second+'s ease-out';
                }

                var leftTarget = this.$slidePage[leftNo];
                var leftHeight = parseFloat(leftTarget.getAttribute('init-height'));
                var rightTarget = this.$slidePage[rightNo];
                var rightHeight = parseFloat(rightTarget.getAttribute('init-height'));

                var height = leftHeight*leftX+rightHeight*rightX;

                this.autoHeightMove(leftTarget,leftHeight,rightTarget,rightHeight,height,transition);
                this.autoHeightInit(height,this.maxHeight,transition);
            }
            if(this.initPosition){
                x = x-this.initPosition;
            }
            target.style[transformProperty] = 'translateX('+x+'px) translateZ(0)';
        }
        if(second!=null){
            target.style[transitionProperty] = transformDash+' '+second+'s ease-out';
            if(this.isAutoScroll){//自动滚动重新开始
                this.autoScroll();
            }
        }else{
            target.style[transitionProperty] = 'none';
        }
    }

    SimpleSlider.prototype.move = function(e){
        /* caution */
        if(this.isFixed){
            var touch = e.touches[0];
            this.moveX = touch.clientX - this.startX;
            if(this.touchstart){
                this.moveY = touch.clientY - this.startY;
                if(Math.abs(this.moveX)>Math.abs(this.moveY)/2){
                    this.moveItem(e);
                    this.touchstart = false;
                    this.preventDefault  = true;
                }else{
                    this.touchstart = false;
                    this.preventDefault  = false;//否则默认为用户想滑动页面
                }
            }else{
                if(this.preventDefault){
                    this.moveItem(e)
                }
            }
        }
        e.stopPropagation();
        if(this.isPreventDefault){
            e.preventDefault();
        }
    }

    SimpleSlider.prototype.end = function(e){
        //touch结束，如果还没触发自动移动，要么自动移动到初始位置，要么自动移动下一个位置
        if(this.isFixed){
            if(this.atStartFunc()&&this.moveX>0){
                //第一幕不能再向左移动
                this.removeListener();
            }else if(this.atEndFunc()&&this.moveX<0){
                //最后一幕不能再向右移动
                this.removeListener();
            }else{
                if(Math.abs(this.moveX)>0&&this.preventDefault){
                    this.isFixed = false;
                    this.cssTransform(this.$slide,-this.index*this.pageWidth,this.duration);
                }
            }
        }
        e.stopPropagation();
        if(this.isPreventDefault){
            e.preventDefault();
        }
    }

    return SimpleSlider;
}));